import unittest

from src.odoo_env_config import api
from src.odoo_env_config.mappers import queue_job


class TestOdooQueueJobMapper(unittest.TestCase):
    def test_empty(self):
        result = queue_job(api.Env())
        self.assertEqual(1, len(result.keys()))
        for key_result, value in result.items():
            if key_result == "ODOO_QUEUE_JOB_ENABLE":
                self.assertEqual("False", value)
                continue
            self.assertIsNone(value, "Value of key [%s] is not None" % key_result)

    def test_not_change(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        value6 = "6"
        value7 = "7"
        value8 = "8"
        to_map = {
            "ODOO_QUEUE_JOB_ENABLE": "True",
            "ODOO_QUEUE_JOB_CHANNELS": value1,
            "ODOO_QUEUE_JOB_SCHEME": value2,
            "ODOO_QUEUE_JOB_HOST": value3,
            "ODOO_QUEUE_JOB_PORT": value4,
            "ODOO_QUEUE_JOB_HTTP_AUTH_USER": value5,
            "ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD": value6,
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST": value7,
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT": value8,
        }
        result = queue_job(api.Env(to_map))
        self.assertEqual(9, len(result.keys()))
        self.assertEqual(value1, result["ODOO_QUEUE_JOB_CHANNELS"])
        self.assertEqual(value2, result["ODOO_QUEUE_JOB_SCHEME"])
        self.assertEqual(value3, result["ODOO_QUEUE_JOB_HOST"])
        self.assertEqual(value4, result["ODOO_QUEUE_JOB_PORT"])
        self.assertEqual(value5, result["ODOO_QUEUE_JOB_HTTP_AUTH_USER"])
        self.assertEqual(value6, result["ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD"])
        self.assertEqual(value7, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST"])
        self.assertEqual(value8, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT"])

    def test_second_value(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        value6 = "6"
        value7 = "7"
        value8 = "8"
        to_map = {
            "ODOO_QUEUE_JOB_ENABLE": "True",
            "ODOO_CONNECTOR_CHANNELS": value1 + "_fake",
            "ODOO_CONNECTOR_SCHEME": value2 + "_fake",
            "ODOO_CONNECTOR_HOST": value3 + "_fake",
            "ODOO_CONNECTOR_PORT": value4 + "_fake",
            "ODOO_CONNECTOR_HTTP_AUTH_USER": value5 + "_fake",
            "ODOO_CONNECTOR_HTTP_AUTH_PASSWORD": value6 + "_fake",
            "ODOO_CONNECTOR_JOBRUNNER_DB_HOST": value7 + "_fake",
            "ODOO_CONNECTOR_JOBRUNNER_DB_PORT": value8 + "_fake",
            "ODOO_QUEUE_JOB_CHANNELS": value1,
            "ODOO_QUEUE_JOB_SCHEME": value2,
            "ODOO_QUEUE_JOB_HOST": value3,
            "ODOO_QUEUE_JOB_PORT": value4,
            "ODOO_QUEUE_JOB_HTTP_AUTH_USER": value5,
            "ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD": value6,
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST": value7,
            "ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT": value8,
        }
        result = queue_job(api.Env(to_map))
        self.assertEqual(17, len(result.keys()))
        self.assertEqual(value1, result["ODOO_QUEUE_JOB_CHANNELS"])
        self.assertEqual(value2, result["ODOO_QUEUE_JOB_SCHEME"])
        self.assertEqual(value3, result["ODOO_QUEUE_JOB_HOST"])
        self.assertEqual(value4, result["ODOO_QUEUE_JOB_PORT"])
        self.assertEqual(value5, result["ODOO_QUEUE_JOB_HTTP_AUTH_USER"])
        self.assertEqual(value6, result["ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD"])
        self.assertEqual(value7, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST"])
        self.assertEqual(value8, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT"])

    def test_priority(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        value6 = "6"
        value7 = "7"
        value8 = "8"
        to_map = {
            "ODOO_QUEUE_JOB_ENABLE": "True",
            "ODOO_CONNECTOR_CHANNELS": value1,
            "ODOO_CONNECTOR_SCHEME": value2,
            "ODOO_CONNECTOR_HOST": value3,
            "ODOO_CONNECTOR_PORT": value4,
            "ODOO_CONNECTOR_HTTP_AUTH_USER": value5,
            "ODOO_CONNECTOR_HTTP_AUTH_PASSWORD": value6,
            "ODOO_CONNECTOR_JOBRUNNER_DB_HOST": value7,
            "ODOO_CONNECTOR_JOBRUNNER_DB_PORT": value8,
        }
        result = queue_job(api.Env(to_map))
        self.assertEqual(17, len(result.keys()))
        self.assertEqual(value1, result["ODOO_QUEUE_JOB_CHANNELS"])
        self.assertEqual(value2, result["ODOO_QUEUE_JOB_SCHEME"])
        self.assertEqual(value3, result["ODOO_QUEUE_JOB_HOST"])
        self.assertEqual(value4, result["ODOO_QUEUE_JOB_PORT"])
        self.assertEqual(value5, result["ODOO_QUEUE_JOB_HTTP_AUTH_USER"])
        self.assertEqual(value6, result["ODOO_QUEUE_JOB_HTTP_AUTH_PASSWORD"])
        self.assertEqual(value7, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_HOST"])
        self.assertEqual(value8, result["ODOO_QUEUE_JOB_JOBRUNNER_DB_PORT"])



import os
import unittest

from odoo_env_config.api import OdooConfig, Env

try:
    ODOO_VERSION = int(float(os.getenv("ODOO_VERSION", "0")))
except ValueError:
    ODOO_VERSION = None





class MultiOdooVersion:
    """
    Contains decorator run function for each Odoo version supported
    See the doc to use it.


    """

    @staticmethod
    def with_args(function):
        def _wrapper(case):
            for version in range(11, 30 + 1):
                with case.subTest(odoo_version=version):
                    function(case, version)

        return _wrapper

    @staticmethod
    def with_env(function):
        """

        Args:
            function:

        Returns:

        """

        def _wrapper(case):
            for version in range(0, 30 + 1):
                with case.subTest(odoo_version=version):
                    function(case, Env(ODOO_VERSION=version))

        return _wrapper

    @staticmethod
    def without_args(function):
        def _wrapper(case):
            for version in range(0, 30 + 1):
                with case.subTest(odoo_version=version):
                    function(case)

        return _wrapper


class SkipUnless:
    env_odoo = unittest.skipUnless(
        ODOO_VERSION,
        f"Not in Odoo Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo11 = unittest.skipUnless(
        ODOO_VERSION == 11,
        f"Not in Odoo 11 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo12 = unittest.skipUnless(
        ODOO_VERSION == 12,
        f"Not in Odoo 12 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    odoo_more11 = unittest.skipUnless(
        ODOO_VERSION >= 12,
        f"Not in Odoo version more than 12, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo13 = unittest.skipUnless(
        ODOO_VERSION == 13,
        f"Not in Odoo 13 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo14 = unittest.skipUnless(
        ODOO_VERSION == 14,
        f"Not in Odoo 14 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo15 = unittest.skipUnless(
        ODOO_VERSION == 15,
        f"Not in Odoo 15 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )
    env_odoo16 = unittest.skipUnless(
        ODOO_VERSION == 16,
        f"Not in Odoo 16 Env, ODOO_VERSION is {ODOO_VERSION or 'not'} supplied",
    )



# -*- coding: utf8 -*-
import unittest

from src.odoo_env_config import entry, mappers, section


class TestOdooConfig(unittest.TestCase):
    def test_all_mappers_are_register(self):
        number_of_mapper = 5
        self.assertEqual(len(mappers.__all__), number_of_mapper)
        self.assertEqual(len(entry.MAPPER), number_of_mapper)
        self.assertSetEqual(
            set(mappers.__all__), {mapper.__name__ for mapper in entry.MAPPER}
        )

    def test_all_section_are_register(self):
        number_of_section = 12
        self.assertEqual(len(section.__all__), number_of_section)
        self.assertEqual(len(entry.CONVERTER), number_of_section)
        self.assertSetEqual(
            set(section.__all__), {mapper.__name__ for mapper in entry.CONVERTER}
        )



import unittest

from src.odoo_env_config import api
from src.odoo_env_config.mappers import redis_session


class TestOdooQueueJobMapper(unittest.TestCase):
    def test_empty(self):
        result = redis_session(api.Env())
        self.assertEqual(1, len(result.keys()))
        for key_result, value in result.items():
            if key_result == "REDIS_SESSION_ENABLE":
                self.assertEqual("False", value)
                continue
            self.assertIsNone(value, "Value of key [%s] is not None" % key_result)

    def test_not_change(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        to_map = {
            "REDIS_SESSION_ENABLE": str(True),
            "REDIS_SESSION_URL": value1,
            "REDIS_SESSION_HOST": value2,
            "REDIS_SESSION_PORT": value3,
            "REDIS_SESSION_DB_INDEX": value4,
            "REDIS_SESSION_PASSWORD": value5,
        }
        result = redis_session(api.Env(to_map))
        self.assertEqual(6, len(result.keys()))
        self.assertEqual("True", result["REDIS_SESSION_ENABLE"])
        self.assertEqual(value1, result["REDIS_SESSION_URL"])
        self.assertEqual(value2, result["REDIS_SESSION_HOST"])
        self.assertEqual(value3, result["REDIS_SESSION_PORT"])
        self.assertEqual(value4, result["REDIS_SESSION_DB_INDEX"])
        self.assertEqual(value5, result["REDIS_SESSION_PASSWORD"])

    def test_priority(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        to_map = {
            "REDIS_SESSION_ENABLE": "True",
            "REDIS_URL": value1 + "_fake",
            "REDIS_HOST": value2 + "_fake",
            "REDIS_PORT": value3 + "_fake",
            "REDIS_DB_INDEX": value4 + "_fake",
            "REDIS_PASSWORD": value5 + "_fake",
            "REDIS_SESSION_URL": value1,
            "REDIS_SESSION_HOST": value2,
            "REDIS_SESSION_PORT": value3,
            "REDIS_SESSION_DB_INDEX": value4,
            "REDIS_SESSION_PASSWORD": value5,
        }
        result = redis_session(api.Env(to_map))
        self.assertEqual(11, len(result.keys()))
        self.assertEqual("True", result["REDIS_SESSION_ENABLE"])
        self.assertEqual(value1, result["REDIS_SESSION_URL"])
        self.assertEqual(value2, result["REDIS_SESSION_HOST"])
        self.assertEqual(value3, result["REDIS_SESSION_PORT"])
        self.assertEqual(value4, result["REDIS_SESSION_DB_INDEX"])
        self.assertEqual(value5, result["REDIS_SESSION_PASSWORD"])

    def test_second_value(self):
        value1 = "1"
        value2 = "2"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        to_map = {
            "REDIS_SESSION_ENABLE": "True",
            "REDIS_URL": value1,
            "REDIS_HOST": value2,
            "REDIS_PORT": value3,
            "REDIS_DB_INDEX": value4,
            "REDIS_PASSWORD": value5,
        }
        result = redis_session(api.Env(to_map))
        self.assertEqual(11, len(result.keys()))
        self.assertEqual("True", result["REDIS_SESSION_ENABLE"])
        self.assertEqual(value1, result["REDIS_SESSION_URL"])
        self.assertEqual(value2, result["REDIS_SESSION_HOST"])
        self.assertEqual(value3, result["REDIS_SESSION_PORT"])
        self.assertEqual(value4, result["REDIS_SESSION_DB_INDEX"])
        self.assertEqual(value5, result["REDIS_SESSION_PASSWORD"])

    def test_not_enable(self):
        value1 = "1"
        value3 = "3"
        value4 = "4"
        value5 = "5"
        to_map = {
            "REDIS_URL": value1,
            "REDIS_SESSION_PORT": value3,
            "REDIS_SESSION_DB_INDEX": value4,
            "REDIS_SESSION_PASSWORD": value5,
        }
        result = redis_session(api.Env(to_map))
        self.assertEqual(5, len(result.keys()))
        self.assertEqual("False", result["REDIS_SESSION_ENABLE"])



from typing_extensions import Self

from .. import api
from .worker_section import WorkersOdooConfigSection


class LimitOdooConfigSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.limit_request = 0
        self.limit_time_cpu = 0
        self.limit_time_real = 0
        self.osv_memory_count_limit = 0
        self.osv_memory_age_limit = 0
        self.limit_memory_hard = 0
        self.limit_memory_soft = 0

    def init(self, env: api.Env) -> Self:
        self.limit_request = env.get_int("LIMIT_REQUEST")
        self.limit_time_cpu = env.get_int("LIMIT_TIME_CPU")
        self.limit_time_real = env.get_int("LIMIT_TIME_REAL")
        self.osv_memory_count_limit = env.get_int("OSV_MEMORY_COUNT_LIMIT")
        self.osv_memory_age_limit = env.get_int("OSV_MEMORY_AGE_LIMIT")
        self.limit_memory_hard = env.get_int("LIMIT_MEMORY_HARD")
        self.limit_memory_soft = env.get_int("LIMIT_MEMORY_SOFT")

        if not self.limit_memory_hard or not self.limit_memory_soft:
            global_limit_memory_hard = env.get_int("GLOBAL_LIMIT_MEMORY_HARD")
            global_limit_memory_soft = env.get_int("GLOBAL_LIMIT_MEMORY_SOFT")
            nb_workers = WorkersOdooConfigSection().init(env).total or 1
            if not self.limit_memory_soft and global_limit_memory_soft:
                self.limit_memory_soft = global_limit_memory_soft // nb_workers
            if not self.limit_memory_hard and global_limit_memory_hard:
                self.limit_memory_hard = global_limit_memory_hard // nb_workers
        return self

    def to_values(self) -> api.OdooCliFlag:
        res = api.OdooCliFlag()
        return res.set_all(
            {
                "limit-request": self.limit_request,
                "limit-time-cpu": self.limit_time_cpu,
                "limit-time-real": self.limit_time_real,
                "limit-memory-hard": self.limit_memory_hard,
                "limit-memory-soft": self.limit_memory_soft,
                "osv-memory-count-limit": self.osv_memory_count_limit,
                "osv-memory-age-limit": self.osv_memory_age_limit,
            }
        )



from typing import List

from typing_extensions import Self

from .. import api
from ..api import OdooCliFlag


class OdooOptSection(api.EnvConfigSection):
    def __init__(self):
        super().__init__()
        self.options: List[OdooCliFlag] = []

    def init(self, curr_env: api.Env) -> Self:
        """ """
        for key, value in curr_env.get_start_with("OPT_ODOO_"):
            self.options.append(OdooCliFlag().set(key.lower(), value))
        return self

    def write_to_config(self, config: api.OdooConfig):
        for option in self.options:
            for key, value in option.items():
                config[key]=value



import unittest

from src.odoo_env_config.api import Env
from src.odoo_env_config.section.db_section import (
    DatabaseOdooConfigSection,
    MaxConnMode,
)


class TestDatabaseOdooConfigSection(unittest.TestCase):
    def test_default(self):
        conf = DatabaseOdooConfigSection()
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertEqual(0, conf.max_conn)
        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)

    def test_global(self):
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_FILTER": "db_filter.*",
                    "DB_NAME": "db_name",
                    "DB_HOST": "db_host",
                    "DB_MAX_CONN": "20",
                    "DB_PORT": "1234",
                    "DB_USER": "db_user",
                    "DB_PASSWORD": "db_password",
                    "LIST_DB": "True",
                }
            )
        )
        self.assertEqual("db_filter.*", conf.filter)
        self.assertEqual("db_host", conf.host)
        self.assertEqual(20, conf.max_conn)
        self.assertEqual("db_name", conf.name)
        self.assertEqual("db_user", conf.user)
        self.assertEqual(1234, conf.port)
        self.assertEqual("db_password", conf.password)
        self.assertTrue(conf.show)

    def test_db_name(self):
        """
        If a <DB_NAME> is filled and no  <DB_FILTER> then <DB_FILTER> is set to <DB_NAME>
        and DatabaseOdooConfigSection#show is set to false
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_NAME": "db_name",
                }
            )
        )
        self.assertEqual("db_name", conf.name)
        self.assertIsNone(conf.filter)
        self.assertTrue(conf.show)

        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertEqual(0, conf.max_conn)

    def test_db_filter(self):
        """
        If <DB_FILTER> is set but no <DB_NAME> then :
        - DatabaseOdooConfigSection#name is None
        - DatabaseOdooConfigSection#show is True
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_FILTER": "db_filter.*",
                }
            )
        )
        self.assertEqual("db_filter.*", conf.filter)
        self.assertTrue(conf.show)
        self.assertIsNone(conf.name)

        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertEqual(0, conf.max_conn)

    def test_db_name_with_show(self):
        """
        If <DB_NAME> is set but no <DB_FILTER> and <SHOW_DB> then:
        - DatabaseOdooConfigSection#name eq <DB_NAME>
        - DatabaseOdooConfigSection#filter eq <DB_NAME> + '.*'
        - DatabaseOdooConfigSection#show is True
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_NAME": "db_name",
                    "LIST_DB": "True",
                }
            )
        )
        self.assertEqual("db_name", conf.name)
        self.assertIsNone(conf.filter)
        self.assertTrue(conf.show)

        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertEqual(0, conf.max_conn)

    def test_max_con_auto(self):
        """
        By default there 3 workers
        If <DB_MAX_CONN_MODE> is set to "AUTO" and a <DB_MAX_CONN> is provided then:
        if <DB_MAX_CONN> superior to AUTO computed value
        - DatabaseOdooConfigSection#max_conn eq <DB_MAX_CONN>
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_MAX_CONN_MODE": MaxConnMode.AUTO.value,
                    "DB_MAX_CONN": 100,
                }
            )
        )
        self.assertEqual(100, conf.max_conn)

        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)

    def test_max_con_auto_workers(self):
        """
        If <DB_MAX_CONN_MODE> is set to "AUTO" then:
        - DatabaseOdooConfigSection#max_conn eq (nb workers total) + (nb workers total) // 2

        We don't care of <DB_MAX_CONN>
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_MAX_CONN_MODE": MaxConnMode.AUTO.value,
                    "WORKER_HTTP": str(100),
                    "DB_MAX_CONN": str(20),
                }
            )
        )
        # 100 + 2 (cron) + 102 // 2 => 150
        self.assertEqual(150, conf.max_conn)

        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)

    def test_max_con_fixed_with_vaue(self):
        """
         By default there 3 workers
         If <DB_MAX_CONN_MODE> is set to "AUTO" then:
         - DatabaseOdooConfigSection#max_conn eq (nb workers total) + (nb workers total) divide by 2

        We ensure the value of <DB_MAX_CONN> is taken
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_MAX_CONN_MODE": MaxConnMode.FIXED.value,
                    "DB_MAX_CONN": 100,
                }
            )
        )
        self.assertEqual(100, conf.max_conn)

        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)

    def test_max_con_fixed_with_less_than_required(self):
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_MAX_CONN_MODE": MaxConnMode.FIXED.value,
                    "DB_MAX_CONN": 20,
                    "WORKER_HTTP": 15,
                }
            )
        )
        self.assertEqual(20, conf.max_conn)

        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)

    def test_max_con_fixed_no_value(self):
        """
        By default there 3 workers
        If <DB_MAX_CONN_MODE> is set to "FIXED" then:
        - DatabaseOdooConfigSection#max_conn eq <DB_MAX_CONN>>

        In this test we don't have <DB_MAX_CONN> so we ensure the system switch back to auto
        """
        conf = DatabaseOdooConfigSection().init(
            Env(
                {
                    "DB_MAX_CONN_MODE": MaxConnMode.FIXED.value,
                }
            )
        )
        self.assertEqual(0, conf.max_conn)

        self.assertIsNone(conf.filter)
        self.assertIsNone(conf.name)
        self.assertIsNone(conf.host)
        self.assertIsNone(conf.user)
        self.assertEqual(0, conf.port)
        self.assertIsNone(conf.password)
        self.assertTrue(conf.show)
